function [pv12,ev12]=p_12(S,m,u,n,Var,Var01,Var0101,Exp,Exp01)

%   This function uses a modification of an algorithm in the paper
%      "Numerical Computation of Multivariate Normal Probabilities", 
%      A. Genz, J. of Comp. Graph. Stat., 1(1992), pp. 141-149.
%
%   The primary references for the numerical integration are 
%    "Randomization of Number Theoretic Methods for Multiple Integration",
%     R. Cranley & T.N.L. Patterson, SIAM J Numer Anal, 13(1976), pp. 904-14.
%   and  
%    "Fast Component-by-Component Construction, a Reprise for Different 
%     Kernels", D. Nuyens & R. Cools. In H. Niederreiter and D. Talay,
%     editors, Monte-Carlo and Quasi-Monte Carlo Methods 2004, 
%     Springer-Verlag, 2006, 371-385.

eps1 = 5e-9;

pv12 = zeros(1,n+1);
ev12 = zeros(1,n+1);
df = zeros(S,n+1);


for i=0:n,
        Sg = zeros(i+2);
        Sg(1,1)= Var((i+1)*(n+1),(i+1)*(n+1)); 
        Sg(2,1)= Var01((i+1)*(n+1),(i+1)*(n+1)); Sg(1,2) = Sg(2,1); Sg(2,2)= Var0101(n+1,i+1); 
        
        if not(i==0),
        Sg(3:i+2,1) = Var((i+1)*(n+1),(n+1):(n+1):i*(n+1))'; Sg(1,3:i+2) = Sg(3:i+2,1)';
        Sg(3:i+2,2) = Var01((i+1)*(n+1),(n+1):(n+1):i*(n+1))'; 
        Sg(2,3:i+2) = Sg(3:i+2,2)';
        Sg(3:i+2,3:i+2) = Var((n+1):(n+1):i*(n+1),(n+1):(n+1):i*(n+1)) + Var((n+1):(n+1):i*(n+1),(n+1):(n+1):i*(n+1))' - diag(diag(Var((n+1):(n+1):i*(n+1),(n+1):(n+1):i*(n+1)))); %Covariance matrix
        end
        
        [ U, D ] = svd(Sg);
        d = sqrt(diag(D));
        nc = i+2;
        if d(nc) < eps1, 
            nc = min( find( d < eps1 ) );
        end
        [ Q, R ] = qr( diag(d(1:nc))*U(:,1:nc)' );
        ch = ( diag(sign(diag(R)))*R )';  %Cholesky for S_i
       
       [q pr] = fstrnk(m,nc-1);  %Lattice Rule 
        q = q/pr;
            
            
        uv = (u - Exp(n+1,i+1))/ch(1,1)*ones(1,m);
        N_1 = Phi2((-Exp01(n+1,i+1)-ch(2,1)*(u-Exp(n+1,i+1))/ch(1,1))/ch(2,2));
        pu = phi(uv)*(1-N_1);
        
        for K = 1:S,
            fi = zeros(1,m); y = zeros(nc-2,m);
            y(1,:) = Phinv(abs(2*mod(q(1)*[1:m]+rand,1) - 1 )*(1-N_1) + N_1);
            fi = pu.*(ch(2,2)*y(1,:)+ ch(2,1)*uv + Exp01(n+1,i+1)*ones(1,m))/ch(1,1);
            
                    bi = 9;
                    ai = -9;
        
                for j = 2 : nc-2, 
                    ys = ch(j+1,1:j)*[uv;y(1:j-1,:)];
                    dc = Phi2( min( ( u - Exp(j - 1 - floor((j-2)/(n+1))*(n+1),floor((j-2)/(n+1)) + 1) - ys )/ch(j+1,j+1), 9 ) );
                    fi = fi.*dc;
                    y(j,:) = min(max(Phinv(abs( 2*mod(q(j)*[1:m]+rand,1) - 1 ).*dc ),-1e30),1e30); % MCQMC
                end
               
                 for j = nc:i+2,
                     
                    ys = ch(j,1:nc-1)*[ uv; y(1:nc-2,:) ];
                      if ch(j,nc) > 0,
                           bi = min( ( u - Exp(j - 1 - floor((j-2)/(n+1))*(n+1),floor((j-2)/(n+1)) + 1) - ys )/ch(j,nc), bi );
                           else
                           ai = max( ( u - Exp(j - 1 - floor((j-2)/(n+1))*(n+1),floor((j-2)/(n+1)) + 1) - ys )/ch(j,nc), ai );
                      end
                 end
                
            df(K,i+1) = mean(fi.*(Phi2(bi) - Phi2(ai))); %Estimation of S_i 
        end
%         Sg = zeros(min(i+2,3));
%         Sg(1,1)= Var((i+1)*(n+1),(i+1)*(n+1)); 
%         Sg(2,1)= Var01((i+1)*(n+1),(i+1)*(n+1)); Sg(1,2) = Sg(2,1); Sg(2,2)= Var0101(n+1,i+1); 
%         
%         if not(i==0),
%         Sg(3,1) = Var((i+1)*(n+1),n+1); Sg(1,3) = Sg(3,1);
%         Sg(3,2) = Var01((i+1)*(n+1),n+1); 
%         Sg(2,3) = Sg(3:3,2);
%         Sg(3,3) = Var((n+1),(n+1)); %Covariance matrix
%         end
%         
%         [ U, D ] = svd(Sg);
%         d = sqrt(diag(D));
%         nc = min(i+2,3);
%         if d(nc) < eps1, 
%             nc = min( find( d < eps1 ) );
%         end
%         [ Q, R ] = qr( diag(d(1:nc))*U(:,1:nc)' );
%         ch = ( diag(sign(diag(R)))*R )';  %Cholesky for S_i
%        
%        [q pr] = fstrnk(m,nc-1);  %Lattice Rule 
%         q = q/pr;
%             
%             
%         uv = (u - Exp(n+1,i+1))/ch(1,1)*ones(1,m);
%         N_1 = Phi2((-Exp01(n+1,i+1)-ch(2,1)*(u-Exp(n+1,i+1))/ch(1,1))/ch(2,2));
%         pu = phi(uv)*(1-N_1);
%         
%         for K = 1:S,
%             fi = zeros(1,m); y = zeros(nc-2,m);
%             y(1,:) = Phinv(abs(2*mod(q(1)*[1:m]+rand,1) - 1 )*(1-N_1) + N_1);
%             fi = pu.*(ch(2,2)*y(1,:)+ ch(2,1)*uv + Exp01(n+1,i+1)*ones(1,m))/ch(1,1);
%             
%                     bi = 9;
%                     ai = -9;
%         
%                 for j = 2 : nc-2, 
%                     ys = ch(j+1,1:j)*[uv;y(1:j-1,:)];
%                     dc = Phi2( min( ( u - Exp(j - 1 - floor((j-2)/(n+1))*(n+1),floor((j-2)/(n+1)) + 1) - ys )/ch(j+1,j+1), 9 ) );
%                     fi = fi.*dc;
%                     y(j,:) = min(max(Phinv(abs( 2*mod(q(j)*[1:m]+rand,1) - 1 ).*dc ),-1e30),1e30); % MCQMC
%                 end
%                
%                  for j = nc:min(i+2,3),
%                      
%                     ys = ch(j,1:nc-1)*[ uv; y(1:nc-2,:) ];
%                       if ch(j,nc) > 0,
%                            bi = min( ( u - Exp(j - 1 - floor((j-2)/(n+1))*(n+1),floor((j-2)/(n+1)) + 1) - ys )/ch(j,nc), bi );
%                            else
%                            ai = max( ( u - Exp(j - 1 - floor((j-2)/(n+1))*(n+1),floor((j-2)/(n+1)) + 1) - ys )/ch(j,nc), ai );
%                       end
%                  end
%                 
%             df(K,i+1) = mean(fi.*(Phi2(bi) - Phi2(ai))); %Estimation of S_i 
%         end

end
pv12 = mean(df);
ev12 = 1/(S*(S-1))*sum((df - ones(S,1)*pv12).^2); ev12 = 3*sqrt(ev12);
end
